#!/bin/bash

CURR_VERSION="1.0.16-beta-111214"
TMPDIR="tmp-cpptesk-toolkit"
ONLY_CHECK="0"
this=`basename "$0"`

usage () {
   cat <<EOF

Usage:
  $this [options]

  This script checks and installs updates for C++TESK Testing ToolKit.

  Options:
    --check                   - check if new version exists without installing
    --install                 - install the latest available version of C++TESK if given (default)
    --reinstall               - reinstall C++TESK with the latest available version
    --help | -h               - show this message

EOF
}

argc=$(( $# ))

for(( i = 0; $i < $argc; i++ ))
do
  case $1 in
    --check )
      ONLY_CHECK="1"
    ;;
    --help | -h )
      usage
      exit 0
    ;;
    --install )
    ;;
    --reinstall )
      CURR_VERSION="version_for_reinstall"
    ;;
    * )
      usage
      exit 0
  esac
  shift
done

mkdir -p $TMPDIR
cd $TMPDIR

wget -q --no-cache "http://forge.ispras.ru/repo/cpptesk-toolkit/tar.gzs/LatestVersion"
RWGET=`find . -name LatestVersion`

if [ -n "$RWGET" ]
then
  LAST_VERSION=`cat LatestVersion`
  if [ $LAST_VERSION == $CURR_VERSION ]
  then
     tput setaf 2
     echo "[OK] No updates are available."
     tput sgr0
  else
     wget -q "http://forge.ispras.ru/repo/cpptesk-toolkit/tar.gzs/cpptesk-toolkit-src-$LAST_VERSION.tar.gz"
     RWGET=`find . -name cpptesk-toolkit-src-$LAST_VERSION.tar.gz`
     if [ -n "$RWGET" ]
     then
       tput setaf 2
       echo "[INFO] C++TESK Testing ToolKit $CURR_VERSION is currently installed."
       echo "[INFO] C++TESK Testing ToolKit $LAST_VERSION is available."
       tput sgr0

       #while :
       #do
         line=""
         if [ $ONLY_CHECK != "1" ]
         then
             echo -n "Would you like to have the new version installed? [y/N] "
             read line
         fi
         case "$line" in
         "y" | "Y" )
           #echo "New version will be installed."
         ;;
         * )
           rm cpptesk-toolkit-src-$LAST_VERSION.tar.gz -f
           tput setaf 3
           if [ $ONLY_CHECK != "1" ]
           then
               echo "[INFO] Stopped by user."
           fi
           tput sgr0
           exit 0
         esac
       #done

       tput setaf 2
       echo "[INFO] Update has been started..."
       tput sgr0
       tar xf cpptesk-toolkit-src-$LAST_VERSION.tar.gz
       rm cpptesk-toolkit-src-$LAST_VERSION.tar.gz -f
       cd cpptesk-toolkit-src-$LAST_VERSION
       ./install.sh
       cd ..
     else
       tput setaf 1
       echo "[FAILED] Check your Internet connection."
       tput sgr0
     fi
  fi
else
  tput setaf 1
  echo "[FAILED] Check your Internet connection."
  tput sgr0
fi

cd ..
rm $TMPDIR -rf

