/** @file multififo_model.h
 * Header for united FIFOs model.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <fifo_media.h>

#include <hw/model.hpp>
#include <fifo_msg.h>

namespace cpptesk {
namespace examples {
namespace fifo {

CPPTESK_MODEL(Multififo) {
public:
    Multififo();
    virtual ~Multififo();

    /// Input interface iface1a to send stimuli for writting into FIFO A.
    CPPTESK_DECLARE_INPUT(iface1a);
    /// Input interface iface1b to send stimuli for writting into FIFO B.
    CPPTESK_DECLARE_INPUT(iface1b);
    /// Input interface iface2 to send stimuli for reading from MultiFIFO.
    CPPTESK_DECLARE_INPUT(iface2);
    /// Output interface iface3 to read results of reading.
    CPPTESK_DECLARE_OUTPUT(iface3);

    /// Check whether interface iface1a is free. @return iface1a.isReady()
    virtual bool isIface1aReady() const;
    /// Check whether interface iface1b is free. @return iface1b.isReady()
    virtual bool isIface1bReady() const;
    /// Check whether interface iface2 is free. @return iface2.isReady()
    virtual bool isIface2Ready() const;

    /// Stimulus "put data into buffer A".
    CPPTESK_DECLARE_STIMULUS(push_msg_a);
    /// Stimulus "put data into buffer B".
    CPPTESK_DECLARE_STIMULUS(push_msg_b);
    /// Stimulus "start reading from MultiFIFO".
    CPPTESK_DECLARE_STIMULUS(pop_msg);
    /// Reaction "read data floating from the MultiFIFO".
    CPPTESK_DECLARE_REACTION(get_pop_msg);

    FIFOMediator fifo0, fifo1, fifo2;
};

}}}
