/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Mikhail Chupilko (chupilko@ispras.ru)
// SYNOPSIS: Reaction arbiters
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/arbiter/base.hpp>

namespace cpptesk {
namespace hw {

////////////////////////////////////////////////////////////////////////////////
// Adaptive Reaction Arbiter
////////////////////////////////////////////////////////////////////////////////

#define CPPTESK_COMPARE_ARBITER ::cpptesk::hw::AdaptiveArbiter

#ifdef HW_SHORT_NAMES
    #define COMPARE_ARBITER CPPTESK_COMPARE_ARBITER
#endif // HW_SHORT_NAMES

class AdaptiveArbiter : public Arbiter
{
public:
    AdaptiveArbiter() {}
    virtual ~AdaptiveArbiter() {}
    
    virtual void appendRequest(const Process &process, const Message &message)
    {
        Request req = { (Process*)&process, (Message*)message.clone() };
        requests.push_back(req);
	messages.push_back((Message*)message.clone());
    }

    virtual bool chooseRequest(const Process &process, const Message &message)
    {
        CPPTESK_ASSERTION(!requests.empty(), "There are no requests to choose");

        for(unsigned int i = 0; i < requests.size(); i++)
        {
            Request req = requests[i];
            if(req.process == &process && req.message->forceCompare(message) == Message::COMPARE_OK)
            {
                 return true;
            }
        }

        return false;
    }

    virtual void removeRequest(const Process &process, const Message &message)
    {
        CPPTESK_ASSERTION(!requests.empty(), "There are no requests to remove");

        for(unsigned int i = 0; i < requests.size(); i++)
        {
            Request req = requests[i];
            if(req.process == &process && req.message->forceCompare(message) == Message::COMPARE_OK)
            {
                requests.erase(requests.begin() + i);
		messages.erase(messages.begin() + i);
                break;
            }
        }
    }

    virtual bool isModelBased() const { return false; }

protected:
    struct Request
    {
        Process *process;
        SharedMessage message;
    };

    /// Queue of reaction requests.
    std::vector<Request> requests;
};

}} // namespace cpptesk::hw

