/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Rule # 2 in C++TESK.ErrorDiagnostics.ru.doc: {[o,o] -> [x,x]}  
//	         Use for normal reactions
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include "hw/message.hpp"
#include "hw/event.hpp"

#include "rule.hpp"
#include "tunit.hpp"
#include "string.h"
#include "runit.hpp"

#include <stdio.h>
#include <string>


namespace cpptesk {
namespace hw {
namespace diagnostics {

class CorrectReactionEliminationRule: 
      public Rule
{
    public:

    CorrectReactionEliminationRule() {}
    virtual ~CorrectReactionEliminationRule() {};

    ReactionSet transform(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2)    // Collapse the reactions
    {
        ReactionSet _transform_out;
    
        CPPTESK_UNUSED_ARG(_reactionUnit_2);
        _reactionUnit_1.useInCollapse();
    
        // TODO: place here CPPTESK_DEBUG_PRINT(DEBUG_INFO, "") instead of printf (debug.hpp)
        
        return _transform_out;
    }

    int match(::cpptesk::hw::Reaction _reaction_1, ::cpptesk::hw::Reaction _reaction_2)
    // Compare the reactions
    {

        int _match;
        CPPTESK_UNUSED_ARG(_reaction_2);
	
        if(_reaction_1.getError() == NORMAL_REACTION)
        {
            _match = EXACT_MATCH;
        }
        else
        {
            _match = FALSE_MATCH;
        }
        
        return _match;
    }

    void weight()
    // Get the weight of operation
    {
    }
    
    void setCollapseRule(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2, int _rule)
    // Set rule number
    {
        CPPTESK_UNUSED_ARG(_reactionUnit_2);
        _reactionUnit_1.setCollapseRule(_rule);
    }

    void distance()
    //  
    {
    }
    
    std::string getRuleDescription()
    {
        return "\n Correct Reaction Elimination Rule - use for NORMAL reaction: {[o:o] => [x:x]}.";
    }

};

}}}
