/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Fuzzy Associative Elimination 
//           (Rule # 7 in C++TESK.ErrorDiagnostics.ru.doc):
//           {[a:c]+[b:a] => [a:a]+[b:c]}
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include "rule.hpp"
#include "tunit.hpp"


#include <stdio.h>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class IncompleteAssociativeEliminationRule: 
      public Rule
{
    public:

    IncompleteAssociativeEliminationRule() {}
    virtual ~IncompleteAssociativeEliminationRule() {};

    ReactionSet transform(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2)    // Collapse the reactions
    {
        ReactionSet _transform_out;
        
        //Message from reaction 1
        const ::cpptesk::hw::Message *_message1Spec;
        const ::cpptesk::hw::Message *_message1Impl;
        //Message from reaction 2
        const ::cpptesk::hw::Message *_message2Impl;
        const ::cpptesk::hw::Message *_message2Spec;
        
        std::string _compareMessage_1, _compareMessage_2;
        _compareMessage_1 = _message1Spec->compare(*_message2Impl);
        _compareMessage_2 = _message2Spec->compare(*_message1Impl);

        int match_result_1 /*TODO , match_result_2*/;
        match_result_1 = _compareMessage_1.size();
        //match_result_2 = _compareMessage_2.size();
        if(match_result_1 == 0)
        {
            _message1Spec = _reactionUnit_1._reaction.getSpecMessage();
            _message2Impl = _reactionUnit_2._reaction.getImplMessage();

            _message1Impl = _reactionUnit_1._reaction.getImplMessage();
            _message2Spec = _reactionUnit_2._reaction.getSpecMessage();
        }
        else
        {
            _message1Spec = _reactionUnit_2._reaction.getSpecMessage();
            _message2Impl = _reactionUnit_1._reaction.getImplMessage();

            _message1Impl = _reactionUnit_2._reaction.getImplMessage();
            _message2Spec = _reactionUnit_1._reaction.getSpecMessage();
        }

        const ::cpptesk::hw::Interface *iface1 = _reactionUnit_1._reaction.where();
        ::cpptesk::hw::Reaction reactionIncorrect = Reaction(INCORRECT_REACTION, *iface1, *_message2Spec, *_message1Impl, 1 /*start time*/, 2 /*stop time*/, "Incorrect reaction from rule#7");
        ::cpptesk::hw::Reaction reactionNormal = Reaction(NORMAL_REACTION, *iface1, *_message1Spec, *_message2Impl, 1 /*start time*/, 2 /*stop time*/, "normal reaction from rule#7");
        
        _transform_out.addReaction(reactionIncorrect);
        _transform_out.addReaction(reactionNormal);
        
        // Set flag
        _reactionUnit_1.useInCollapse();
        _reactionUnit_2.useInCollapse();
       
        // TODO: place here CPPTESK_DEBUG_PRINT(DEBUG_INFO, "") instead of printf (debug.hpp)
        
        return _transform_out;
    }

    int match(::cpptesk::hw::Reaction _reaction_1, ::cpptesk::hw::Reaction _reaction_2)
    // Compare the reactions
    {
        int _match = FALSE_MATCH;

        if((_reaction_1.getError() == INCORRECT_REACTION)&&(_reaction_2.getError() == INCORRECT_REACTION))
        {
            // Message from reaction 1
            const ::cpptesk::hw::Message *_message1Spec = _reaction_1.getSpecMessage();
            const ::cpptesk::hw::Message *_message1Impl = _reaction_1.getImplMessage();
            // Message from reaction 2
            const ::cpptesk::hw::Message *_message2Spec = _reaction_2.getSpecMessage();
            const ::cpptesk::hw::Message *_message2Impl = _reaction_2.getImplMessage();
    
            std::string _compareMessage_1, _compareMessage_2;
            _compareMessage_1 = _message1Spec->compare(*_message2Impl);
            _compareMessage_2 = _message2Spec->compare(*_message1Impl);

            int match_result_1, match_result_2;
            match_result_1 = _compareMessage_1.size();
            match_result_2 = _compareMessage_2.size();
            if((match_result_1 == 0)||(match_result_2 == 0))
            {
                _match = EXACT_MATCH;
            }
       }
         
        return _match;
    }

    void weight()
    // Get the weight of operation
    {
    }
    
    void setCollapseRule(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2, int _rule)
    // Set rule number
    {
        _reactionUnit_1.setCollapseRule(_rule);
        _reactionUnit_2.setCollapseRule(_rule);
    }

    void distance()
    //  
    {
    }
    
    std::string getRuleDescription()
    {
        return "\n Spec Intersection Elimination Rule - use for 2 INCORRECT reactions: {[a:c]+[b:a] => [a:a]+[b:c]}.";
    }

};

}}}
