/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include "hw/event.hpp"
#include <hw/message.hpp>
#include "runit.hpp"

#include <vector>


namespace cpptesk {
namespace hw {
namespace diagnostics {

class ReactionSet 
{
    public: 
    const static int REACTION_SET_NOT_EMPTY = 0; 
    const static int REACTION_SET_EMPTY = 1; 
    
    std::vector<ReactionUnit> _reactionVector;
    
    ReactionSet()
    {  //Empty
    }

    ~ReactionSet()
    {  //Empty
    }

    int isEmpty()
    {   
        int _empty;
        if(_reactionVector.size() == 0)
        {
            _empty = REACTION_SET_EMPTY; // True
        }
        else
        {
            _empty = REACTION_SET_NOT_EMPTY; // False
        }
        return _empty;
    }

    void addReaction(const ::cpptesk::hw::Reaction &_reaction)
    {
        ReactionUnit reactionUnit(_reaction);
        this->addReaction(reactionUnit);
    }

    void addReaction(const ReactionUnit &_reactionUnit)
    {
        _reactionVector.push_back(_reactionUnit);
    }

    void addReaction(ReactionSet &_reactionSet, int _parent1, int _parent2)
    {
        for(unsigned int i=0; i<_reactionSet._reactionVector.size(); i++)
        {
            _reactionSet._reactionVector[i].setParent(_parent1, _parent2);
            this->addReaction(_reactionSet._reactionVector[i]);
        }
    }

    void addReaction(ReactionSet &_reactionSet)
    {
        this->addReaction(_reactionSet, -1, -1);
    }

    void initialization(const std::vector<Reaction> &_reaction)
    {
        for(unsigned int i=0; i<_reaction.size(); i++)
        {
           this->addReaction(_reaction[i]);
        }
    }

    void clear()
    {
        _reactionVector.clear();
    }
};

}}}
