/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Union Elimination 
//           (Rule # 4 in C++TESK.ErrorDiagnostics.ru.doc):
//           {[a:x]+[x:a] => [a:a]+[x:x]}
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include "rule.hpp"
#include "tunit.hpp"

#include <stdio.h>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class UnionEliminationRule: 
      public Rule
{
    public:

    UnionEliminationRule() {}
    virtual ~UnionEliminationRule() {};

    ReactionSet transform(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2)    // Collapse the reactions
    {
        ReactionSet _transform_out;
        
        const ::cpptesk::hw::Message *_message1Spec;
        const ::cpptesk::hw::Message *_message2Impl;
        
        if((_reactionUnit_1._reaction.getError() == MISSING_REACTION)&&(_reactionUnit_1._reaction.getError() == UNEXPECTED_REACTION))
        {
            // Message from reaction 1
            _message1Spec = _reactionUnit_1._reaction.getSpecMessage();
            // Message from reaction 2
            _message2Impl = _reactionUnit_2._reaction.getImplMessage();
        }
        else
        {
            // Message from reaction 1
            _message1Spec = _reactionUnit_2._reaction.getSpecMessage();
            // Message from reaction 2
            _message2Impl = _reactionUnit_1._reaction.getImplMessage();
        }
        
        const ::cpptesk::hw::Interface *iface1 = _reactionUnit_1._reaction.where();
        ::cpptesk::hw::Reaction reactionNormal = Reaction(NORMAL_REACTION, *iface1, *_message1Spec, *_message2Impl, 1 /*start time*/, 2 /*stop time*/, "normal reaction from rule#4");
        
        _transform_out.addReaction(reactionNormal);
        
        // Set flag
        _reactionUnit_1.useInCollapse();
        _reactionUnit_2.useInCollapse();
       
        // TODO: place here CPPTESK_DEBUG_PRINT(DEBUG_INFO, "") instead of printf (debug.hpp)
        
        return _transform_out;
    }

    int match(::cpptesk::hw::Reaction _reaction_1, ::cpptesk::hw::Reaction _reaction_2)
    // Compare the reactions
    {
        int _match = FALSE_MATCH;

        if((_reaction_1.getError() == MISSING_REACTION)&&(_reaction_2.getError() == UNEXPECTED_REACTION))
        {
            // Message from reaction 1
            const ::cpptesk::hw::Message *_message1Spec = _reaction_1.getSpecMessage();
            // Message from reaction 2
            const ::cpptesk::hw::Message *_message2Impl = _reaction_2.getImplMessage();
    
            std::string _compareMessage;
            _compareMessage = _message1Spec->compare(*_message2Impl);
            
            int match_result;
            match_result = _compareMessage.size();
            
            if(match_result == 0)
            {
                _match = EXACT_MATCH;
            }
        }

        if((_reaction_2.getError() == MISSING_REACTION)&&(_reaction_1.getError() == UNEXPECTED_REACTION))
        {
            // Message from reaction 1
            const ::cpptesk::hw::Message *_message1Impl = _reaction_1.getImplMessage();
            // Message from reaction 2
            const ::cpptesk::hw::Message *_message2Spec = _reaction_2.getSpecMessage();
            
            std::string _compareMessage;
            _compareMessage = _message1Impl->compare(*_message2Spec);

            int match_result;
            match_result = _compareMessage.size();

            if(match_result == 0)
            {
                _match = EXACT_MATCH;
            }
        }
        
        return _match;
    }

    void weight()
    // Get the weight of operation
    {
    }
    
    void setCollapseRule(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2, int _rule)
    // Set rule number
    {
        _reactionUnit_1.setCollapseRule(_rule);
        _reactionUnit_2.setCollapseRule(_rule);
    }

    void distance()
    //  
    {
    }
    
    std::string getRuleDescription()
    {
        return "\n Union Elimination Rule - use for MISSING and UNEXPECTED reactions: {[a:x]+[x:a] => [a:a]+[x:x]}.";
    }

};

}}}
