/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Test sequence tracer 
///////////////////////////////////////////////////////////////////////////////

#include <hw/defs.hpp>

#include <hw/message.hpp>
#include <hw/time.hpp>
#include <hw/test_tracer.hpp>

#include <string>

namespace cpptesk {
namespace hw {

Tracer::~Tracer()
{
    if(_test_sequence_stream)
    {
        _test_sequence_stream->close();
        delete _test_sequence_stream;
    }
}

void Tracer::setTraceFile(const char *filename)
{
    if(_test_sequence_stream)
    {
        _test_sequence_stream->close();
        delete _test_sequence_stream;
    }

    _test_sequence_stream = new std::ofstream(filename);
}

void Tracer::trace
(
    cycle_t time,
    const std::string &object_name,
    const std::string &stimulus_name,
    const std::string &iface_name,
    const std::string &message_name,
    const Message &message_itself
)
{
    if(!_test_sequence_stream)
        { return; }

    std::ostream &out = *_test_sequence_stream;

    if(_latest_record_time < time)
        { out << "CPPTESK_DELAY(" << std::dec << (time - _latest_record_time) << ");" << std::endl; }

    out << "{" << std::endl;
    // Declare a message
    out << "    " << message_itself.getName() << " " << message_name << ";" << std::endl;

    // Initialize the fields
    for(Message::const_iterator i = message_itself.begin(); i != message_itself.end(); i++)
    {
	const Field<> &field = message_itself.**i;

        out << "    " << message_name << "." << field.getShortName() << " = 0x"
            << std::hex << field.get() << ";" << std::endl;
    }

    // Call the stimulus
    out << "    CPPTESK_CALL_STIMULUS_OF(" << object_name << ", " << stimulus_name << ", "
        << iface_name << ", " << message_name << ");" << std::endl << std::endl;

    out << "}" << std::endl;

    _latest_record_time = time;
}

}} // cpptesk::hw

