/** @file ccp_scenario.h
 *
 * Scenario for verifying the cache coherence protocol.
 *
 * @author Alexander Kamkin (kamkin@ispras.ru)
 */

#pragma once

#include <ts/scenario.hpp>

#include <ccp_model.h>

using namespace cpptesk::ts;
using namespace cpptesk::hw;
using namespace cpptesk::tracer;

namespace cpptesk {
namespace examples {
namespace ccp {

SCENARIO(CacheCoherenceScenario)
{
public:
    typedef IntAbcCtx Ctx;

    static const unsigned N = 8;

    CacheCoherenceScenario(CacheCoherenceModel::state_abstraction_t state_abstraction,
        CacheCoherenceModel::trans_abstraction_t trans_abstraction):
	_model(N, state_abstraction, trans_abstraction)
    {
        setup("CacheCoherenceScenario",
            &CacheCoherenceScenario::init,
            &CacheCoherenceScenario::finish,
            &CacheCoherenceScenario::getState);

        SCENARIO_METHOD(CacheCoherenceScenario::read);
        SCENARIO_METHOD(CacheCoherenceScenario::write);
        SCENARIO_METHOD(CacheCoherenceScenario::replace);
        SCENARIO_METHOD(CacheCoherenceScenario::reset);
    }

    virtual ~CacheCoherenceScenario() {}

    virtual bool init(int argc, char ** argv);
    virtual void finish();
    virtual std::string getState();

    bool read   (Ctx &ctx);
    bool write  (Ctx &ctx);
    bool replace(Ctx &ctx);
    bool reset  (Ctx &ctx);

protected:
    CacheCoherenceModel _model;
};

class VectorStateScenario: public CacheCoherenceScenario
{
public:
    VectorStateScenario():
        CacheCoherenceScenario(CacheCoherenceModel::VECTOR, CacheCoherenceModel::INDEX)
    {
        setup("VectorStateScenario",
            &CacheCoherenceScenario::init,
            &CacheCoherenceScenario::finish,
            &CacheCoherenceScenario::getState);
    }

    virtual ~VectorStateScenario() {}
};

class MultiSetStateScenario: public CacheCoherenceScenario
{
public:
    MultiSetStateScenario():
        CacheCoherenceScenario(CacheCoherenceModel::MULTISET, CacheCoherenceModel::KIND)
    {
        setup("MultiSetStateScenario",
            &CacheCoherenceScenario::init,
            &CacheCoherenceScenario::finish,
            &CacheCoherenceScenario::getState);
    }

    virtual ~MultiSetStateScenario() {}
};

class SetStateScenario: public CacheCoherenceScenario
{
public:
    SetStateScenario():
        CacheCoherenceScenario(CacheCoherenceModel::SET, CacheCoherenceModel::KIND)
    {
        setup("SetStateScenario",
            &CacheCoherenceScenario::init,
            &CacheCoherenceScenario::finish,
            &CacheCoherenceScenario::getState);
    }

    virtual ~SetStateScenario() {}
};

}}} // namespace cpptesk::examples::ccp

