/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Fuzzy Union Elimination 
//           (Rule # 10 in C++TESK.ErrorDiagnostics.ru.doc):
//           {[a:x]+[x:b] => [a:b]+[x:x]}
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include "rule.hpp"
#include "tunit.hpp"

#include <stdio.h>
#include <hw/message.hpp>
#include "../message.hpp"

namespace cpptesk {
namespace hw {
namespace diagnostics {

class FuzzyUnionEliminationRule: 
      public Rule
{
    public:

    FuzzyUnionEliminationRule() {}
    virtual ~FuzzyUnionEliminationRule() {};

    ReactionSet transform(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2)    // Collapse the reactions
    {
        ReactionSet _transform_out;
        
        const ::cpptesk::hw::Message *_message1Spec;
        const ::cpptesk::hw::Message *_message2Impl;
        
        if((_reactionUnit_1._reaction.getError() == MISSING_REACTION)&&(_reactionUnit_1._reaction.getError() == UNEXPECTED_REACTION))
        {
            // Message from reaction 1
            _message1Spec = _reactionUnit_1._reaction.getSpecMessage();
            // Message from reaction 2
            _message2Impl = _reactionUnit_2._reaction.getImplMessage();
        }
        else
        {
            // Message from reaction 1
            _message1Spec = _reactionUnit_2._reaction.getSpecMessage();
            // Message from reaction 2
            _message2Impl = _reactionUnit_1._reaction.getImplMessage();
        }
        
        const ::cpptesk::hw::Interface *iface1 = _reactionUnit_1._reaction.where();
        ::cpptesk::hw::Reaction reactionNormal = Reaction(INCORRECT_REACTION, *iface1, *_message1Spec, *_message2Impl, 1 /*start time*/, 2 /*stop time*/, "Incorrect reaction from rule#10");
        
        _transform_out.addReaction(reactionNormal);
        
        // Set flag
        _reactionUnit_1.useInCollapse();
        _reactionUnit_2.useInCollapse();
       
        // TODO: place here CPPTESK_DEBUG_PRINT(DEBUG_INFO, "") instead of printf (debug.hpp)
        
        return _transform_out;
    }

    int match(::cpptesk::hw::Reaction _reaction_1, ::cpptesk::hw::Reaction _reaction_2)
    // Compare the reactions
    {
        int _match = FALSE_MATCH;

        if((_reaction_1.getError() == MISSING_REACTION)&&(_reaction_2.getError() == UNEXPECTED_REACTION))
        {
            // Message from reaction 1
            const ::cpptesk::hw::Message *_message1Spec = _reaction_1.getSpecMessage();
            // Message from reaction 2
            const ::cpptesk::hw::Message *_message2Impl = _reaction_2.getImplMessage();
    
           // std::string _compareMessage;
           // _compareMessage = _message1Spec->compare(*_message2Impl);
            
            if(_message1Spec->size() == _message2Impl->size())
            {
                _match = FUZZY_MATCH;
                //const Message &_message1Spec1 = _message1Spec;
                //const Message &_message2Impl1 = _message2Impl;
                for(::cpptesk::hw::Message::const_iterator i = _message1Spec->begin(), j = _message2Impl->begin();
                    i != _message1Spec->end(); i++, j++)
                {
                    const Field<> &field_Spec = (*_message1Spec).**i;
                    const Field<> &field_Impl = (*_message2Impl).**j;

                    if(field_Spec.get() == field_Impl.get())
                    {
                        _match--;
                    }
                }
            }
        }

        if((_reaction_2.getError() == MISSING_REACTION)&&(_reaction_1.getError() == UNEXPECTED_REACTION))
        {
            // Message from reaction 1
            const ::cpptesk::hw::Message *_message1Impl = _reaction_1.getImplMessage();
            // Message from reaction 2
            const ::cpptesk::hw::Message *_message2Spec = _reaction_2.getSpecMessage();
            
           if(_message1Impl->size() == _message2Spec->size())
            {
                _match = FUZZY_MATCH;
                for(::cpptesk::hw::Message::const_iterator i = _message1Impl->begin(), j = _message2Spec->begin();
                    i != _message1Impl->end(); i++, j++)
                {
                    const Field<> &field_Impl = (*_message1Impl).**i;
                    const Field<> &field_Spec = (*_message2Spec).**j;

                    if(field_Spec.get() == field_Impl.get())
                    {
                        _match--;
                    }
                }
            }
        }
        
        return _match;
    }

    void weight()
    // Get the weight of operation
    {
    }
    
    void setCollapseRule(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2, int _rule)
    // Set rule number
    {
        _reactionUnit_1.setCollapseRule(_rule);
        //_reactionUnit_1.useInCollapse();
        _reactionUnit_2.setCollapseRule(_rule);
        //_reactionUnit_2.useInCollapse();
    }

    void distance()
    //  
    {
    }
    
    std::string getRuleDescription()
    {
        return "\n Fuzzy Union Elimination Rule - use for MISSING and UNEXPECTED reactions: {[a:x]+[x:b] => [a:b]+[x:x]}.";
    }

};

}}}
